; packets are in a lazy sequence of promises
(def packets (repeatedly promise))

; when a packets arrive, the corresponding promise is realised
(defn put-packet [n content]
  (deliver (nth packets n) content))

; a process that take each packet as long as it is available and all
; its predecessesors have been realised
(defn getter []
  (future
    (doseq [packet (map deref packets)]
      (println (str "*** GETTER: " packet)))))

; simulate sender

; split in words
(defn get-words [text] (re-seq #"\w+" text))

; process that randomly sends the words  until all have been
; successfully sent
(defn send-words [words len]
  (future
  (loop [values-to-send len]
    (if (> values-to-send 0)
      (let [n (rand-int len)
            word (nth words n)]
        
        (if (nil? (put-packet n word))
          (do
            (println (str "*** SENDER: \"" word "\" already sent"))
              (recur values-to-send)
              )
          (do
            (println (str "*** SENDER: \"" word "\" successfully sent"))
            (recur (dec values-to-send)))
          )
        )))))

(defn send-str [str]
  (let [words (get-words str)
        len   (count words)]
    (send-words words len)))


(def static-sender
  (delay
   (future
     (doseq []
       (put-packet 4 "let's")
       (put-packet 2 "getting")
       (put-packet 1 "is")
       (put-packet 1 "is")
       (put-packet 6 "home")
       (put-packet 5 "go")
       (put-packet 3 "late")
       (put-packet 0 "It")
       )
     )
  )

)
